//Basic Tensile Testing Machine Code By Xieshi aka CrazyBlackStone
//Project published under CC-BY-NC-SA license for design contest
 
//Need HX711 library https://github.com/bogde/HX711
#include <HX711.h>
 
//pin definitions
const int stepPin = 2;
const int dirPin = 3;
const int DTPin = A0;
const int SCKPin = A1;
 
//calibration setup: if you know your load cell's calibration factor, uncomment and change the scale.set_scale value in the setup function. If you don't want to calibrate the load cell, comment this out.
// #define calibration
 
//set this to the gear ratio of the stepper motor you're using
#define gearRatio 100
//set this to the lead screw pitch of the stepper motor you're using
#define leadScrewPitch 8
//don't change this unless you want to change the speed
#define speedMultiplier 25
 
//parameters
bool testStart = false;
bool moveStepper = false;
float fastSpeed = 200 * 1 * gearRatio / leadScrewPitch / 60 * speedMultiplier;
const float slowMeasurementDelay = 1 * 1000000;
const float fastMeasurementDelay = 0.15 * 1000000;
float measurementDelay = slowMeasurementDelay;
unsigned long lastMeasurement = 0;
unsigned long lastStep = 0;
 
HX711 scale;
 
void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  Serial.println("INITIALIZING");
  scale.begin(DTPin, SCKPin);
 
  #ifndef calibration
  scale.set_scale(2280.f);                      // this value is obtained by calibrating the scale with known weights details in https://github.com/bogde/HX711
  #endif
 
  scale.tare();
 
  pinMode(dirPin, OUTPUT);
  pinMode(stepPin, OUTPUT);
  digitalWrite(dirPin, LOW);
  digitalWrite(stepPin, LOW);
 
  Serial.println("INITIALIZATION COMPLETE");
}
 
void loop() {
  // put your main code here, to run repeatedly:
  String inputString;
  bool serialAvailable = false;
  while (Serial.available())
  {
    serialAvailable = true;
    inputString = Serial.readString();
    inputString.toLowerCase();
  }
  if (serialAvailable)
  {
    if (inputString == "start")
    {
      startTest();
    }
    else if (inputString == "stop")
    {
      stopTest();
    }
    else if (inputString == "tare")
    {
      Serial.println();
      Serial.println("-SCALE TARE-");
      Serial.println();      
      scale.tare();
    }
    else if (inputString == "up")
    {
      Serial.println();
      Serial.println("-MOVING UP-");
      Serial.println();
      moveStepper = true;
      digitalWrite(dirPin, LOW);
    }
    else if (inputString == "down")
    {
      Serial.println();
      Serial.println("-MOVING DOWN-");
      Serial.println();
      moveStepper = true;
      digitalWrite(dirPin, HIGH);
    }
    else
    {
      Serial.println("Unknown Command");
    }
  }
 
  if ((micros() - lastMeasurement) >= measurementDelay)
  {
    float measurement = scale.get_units();
    Serial.println(measurement);
    lastMeasurement = micros();
  }
 
  if (moveStepper)
  {
    if ((micros() - lastStep) >= 1000000. / fastSpeed)
    {
      digitalWrite(stepPin, HIGH);
      delayMicroseconds(10);
      digitalWrite(stepPin, LOW);
      delayMicroseconds(10); 
      lastStep = micros();
    }
  }
}
 
 
void startTest()
{
  Serial.println();
  Serial.println("-TEST START-");
  Serial.println();
  digitalWrite(dirPin, LOW);
  measurementDelay = fastMeasurementDelay;
  moveStepper = true;
}
 
void stopTest()
{
  Serial.println();
  Serial.println("-STOP-");
  Serial.println();
  measurementDelay = slowMeasurementDelay;
  moveStepper = false;
}
